package com.citydo.webrtcspringboot.controller;

import com.citydo.webrtcspringboot.service.RoomService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

@RestController
@Slf4j
public class IndexController {

    private static String ipOfInet4Address;

    private static final String IP_CODE = "127.0.0.1";

    //拿到本机在wifi中的局域网ip
    static {
        // 获得本机的所有网络接口
        Enumeration<NetworkInterface> naifs;
        try {
            naifs = NetworkInterface.getNetworkInterfaces();
            while (naifs.hasMoreElements()) {
                NetworkInterface nif = naifs.nextElement();
                // 获得与该网络接口绑定的 IP 地址，一般只有一个
                Enumeration<InetAddress> addresses = nif.getInetAddresses();
                while (addresses.hasMoreElements()) {
                    InetAddress addr = addresses.nextElement();
                    // 获取IPv4 地址
                    if (addr instanceof Inet4Address) {
                        ipOfInet4Address = addr.getHostAddress();
                        log.info("网卡接口名称：{}",nif.getName());
                        log.info("网卡接口地址：{}",addr.getHostAddress());
                    }
                }
            }
        } catch (SocketException e) {
            log.error("获取数据失败：{}",e);
        }
    }

    @Value("${server.port}")
    private Integer port;

    @Autowired
    private RoomService roomService;

    @GetMapping("/getWebSocketUrl")
    public Map<String, String> getIpAddress(HttpServletRequest request) {
        Map<String, String> result = new HashMap<>(1);
        if(IP_CODE.equals(request.getRemoteAddr())){
            //本地访问
            result.put("url", "wss:"+request.getRemoteAddr()+":"+port+ "/websocket");
        }else{
            //服务IP访问
            result.put("url", "wss:" + ipOfInet4Address +":"+port+ "/websocket");
        }
        return result;
    }

    @GetMapping("/queryCountInRoom")
    public Map<String, String> queryCountInRoom(String roomId) {
        Map<String, String> result = new HashMap<>(1);
        result.put("count", String.valueOf(roomService.queryCountInRoom(roomId)));
        return result;
    }
}
